/* -----------------------------------------------------------------------
// Copyright: Dale Schinstock, 2013

The DigitalOut interface is used to configure and control a digital output pin.
CAUTION: Max output current is limited to 40 mA for each pin. Need to confirm.

EXAMPLE: Alternative flashing

#include "DigitalOut.h"

int main(void)
{
  DigitalOut pinPB11(PB11);
  
  while(1) {
    
    pinPB11.write(1);
    
    // Insert delay
    delay(1.0);
    
    pinPB11.write(0);
    
    // Insert delay
    delay(1.0);    
  }
}
-------------------------------------------------------------------------*/

#ifndef DIGITAL_OUT_H
#define DIGITAL_OUT_H

#include "stm32f4xx_gpio.h"

// These are the names for the output pins. 

typedef enum 
{
  PB11,
  PD10,
  PD0,
  PD1,
  PD2,
  PD3
} myDigitalOut_TypeDef;

// You can specify the operating output Otype to be:
//  GPIO_OType_PP
//  GPIO_OType_OD
// Default is PP
    
// You can define the operating Pull-up/Pull down to be:     
//  GPIO_PuPd_NOPULL
//  GPIO_PuPd_UP
//  GPIO_PuPd_DOWN
// Default is NOPULL  

class DigitalOut
{
  public:
    //constructors
    //DigitalOut(myDigitalOut_TypeDef pin);
    explicit DigitalOut(myDigitalOut_TypeDef pin, 
               GPIOOType_TypeDef Otype = GPIO_OType_PP,
               GPIOPuPd_TypeDef PuPd = GPIO_PuPd_NOPULL,uint8_t initValue=0);

    //destructor
    virtual ~DigitalOut();

    //functions
    int read(); // Returns 0 or 1 (off or on)
    void write(int value); // Turns off or on (value = 0 or 1)

  private:
    
    //The pin
    uint16_t d_GPIO_PIN;
    uint32_t d_RCC_AHB1Periph;
    GPIO_TypeDef* d_PORT;    
};

#endif