function mlolplots(data,Iscale);
% Copyright: Dale Schinstock, 2013

%Uses data generated by the motorlab control software in openloop.
%If an "Iscale" argument is supplied then the commanded 
%current values are scaled by the Iscale value in the plots.
%
%example: mlolplots(data);  Does not scale the current command.
%
%example: mlolplots(data,Iscale);  Multiplies commanded current values by Iscale.


%Written by DES 8/14/13

TIME=data(:,1); Command=data(:,2); Theta1=data(:,3); Theta2=data(:,4);
Speed1=data(:,5); Speed2=data(:,6); U=data(:,7); I=data(:,8); Extra=data(:,9);

if nargin==2;
   figure(1); 
   plot(TIME,Theta1,TIME,Theta2,TIME,U*Iscale);
   title('Angular Position and Scaled Current Command vs. Time');
   xlabel('Time (sec)');
   label=sprintf('Angle (deg) and Current*%g (Amp)',Iscale);
   ylabel(label);
   legend('Theta1','Theta2','Cur Command');

   figure(2); 
   plot(TIME,Speed1,TIME,Speed2,TIME,U*Iscale);
   title('Angular Speed and Scaled Current Command vs. Time');
   xlabel('Time (sec)');
   label=sprintf('Anglular Speed (RPM) and Current (Amp*%g)',Iscale); 
   ylabel(label);
   legend('Speed1','Speed2','Cur Command');
else
   figure(1); 
   plot(TIME,Theta1,TIME,Theta2,TIME,U);
   title('Angular Position and Current Command vs. Time');
   xlabel('Time (sec)');
   label=sprintf('Angle (deg) and Current (Amp)');
   ylabel(label);
   legend('Theta1','Theta2','Cur Command');

   figure(2); 
   plot(TIME,Speed1,TIME,Speed2,TIME,U);
   title('Angular Speed and Current Command vs. Time');
   xlabel('Time (sec)');
   label=sprintf('Anglular Speed (RPM) and Current (Amp)');
   ylabel(label);
   legend('Speed1','Speed2','Cur Command');
end

figure(3);
plot(TIME,U,TIME,I);
title('Motor Current vs. Time');
xlabel('Time (sec)');
ylabel('Motor Current (Amp)');
legend('Current Command','Measured Current');

% The following is old code was used to plot the current on one
% y scale (right side of plot) and the motion variables on the 
% other y scale (left side) -- it doesn't zoom well

% figure(4)
% leftYs=[Theta1 Theta2];
% [AX,H1,H2]=plotyy(TIME,leftYs,TIME,Command); 
% axes(AX(1));
% title('Angular Position and Current Command vs. Time');
% xlabel('Time (sec)');
% ylabel('Angular Position (deg)');
% H=[H1; H2]; legend(H,'Theta1','Theta2','Command');
% set(H,'ButtonDownFcn','plotscan;')
% axes(AX(2));
% ylabel('Current Command (Amp)');
% 
% figure(5)
% leftYs=[Speed1 Speed2];
% [AX,H1,H2]=plotyy(TIME,leftYs,TIME,Command); 
% axes(AX(1));
% title('Angular Speed and Current Command vs. Time');
% xlabel('Time (sec)');
% ylabel('Angular Speed (RPM)');
% H=[H1; H2]; legend(H,'Speed1','Speed2','Command');
% set(H,'ButtonDownFcn','plotscan;')
% axes(AX(2));
% ylabel('Current Command (Amp)');
