% Copyright: Dale Schinstock, 2013

% gloTalk Object format:
% ????
                 
classdef GloTalkClass < handle
  properties 
    serialPort = serial('COM10','BaudRate',9600,'DataBits',8,'Parity','None','FlowControl','None','StopBits',1);
    numMdBytes = 4;
    CRLF = uint8([13 10]);
  end
  methods
    function this = GloTalkClass()
        this.serialPort.InputBufferSize = 100000;
        fopen(this.serialPort);
    end
    function delete(this)
        fclose(this.serialPort);
    end
    function success = sendObject(this,obj,instance)
        if((instance<1)||(instance>obj.md.numInstance))
            error('object instance out of bounds')
        end
        instance = uint16(instance);
        
        bytes = ['<{(' obj.md.id typecast(instance,'uint8') obj.md.numBytes];

        datacell = struct2cell(obj.data(instance));
        for i=1:length(datacell)
            bytes = [bytes typecast(datacell{i},'uint8')];
        end
        bytes = [bytes this.CRLF];

        flushinput(this.serialPort);
        fwrite(this.serialPort,bytes);  %this blocks till sent, timeout doesn't seem to work

        %now recieve the object  - slave acks by returning the object
        success = this.receiveObject(obj,instance);
    end % sendObject
    function success = getObject(this,obj,instance)
        flushinput(this.serialPort);

        %first send request
        if((instance<0)||(instance>obj.md.numInstance))
            error('object instance out of bounds')
        end
        instance = uint16(instance);
        bytes = ['<{(' 0 typecast(instance,'uint8') 1 obj.md.id this.CRLF];  % request data is req'd objID
        % gen crc ?
        fwrite(this.serialPort,bytes);
        
        %now recieve the object
        if (instance == 0)                   % all instances were requested
            success=this.receiveAllObjects(obj);
        else
            success = this.receiveObject(obj,instance);
        end
    end %getObject
    function success = receiveObject(this,obj,instance)
        wait=1; totNumBytes = double(obj.md.numBytes) + 9;
        tstart = tic;
        while(wait==1)
            if (toc(tstart) > 2)               % timout after 2 second
                wait=0;
                flushinput(this.serialPort);
                display('glo recieve timeout');
                success=0;
            end
            if (this.serialPort.BytesAvailable >= totNumBytes)
               data = uint8(fread(this.serialPort,totNumBytes)'); 
               % check crc ?
               if ~(strcmp(char(data(1:3)),'<{(')||data(4) ~= obj.md.id)||...
                       (typecast(data(5:6),'uint16') ~= instance)||...
                       (data(end-1) ~= 13)||(data(end) ~= 10)
                    data = this.attemptRecovery(data,obj,instance,totNumBytes);
                    if isempty(data)
                        success=0; return;
                    end
               end
               obj.data(instance)=obj.parse(data(8:end-2));
               wait =0;
               success = 1;
            end
        end
    end % recieveObject
    function success = receiveAllObjects(this,obj)
        wait=1; totNumBytes = (double(obj.md.numBytes) + 9)*double(obj.md.numInstance);
        tstart = tic;
        while(wait==1)
            if (toc(tstart) > 3)               % timout after 3 second
                wait=0;
                flushinput(this.serialPort);
                display('glo recieve all timeout');
                success=0;
            end
            if (this.serialPort.BytesAvailable >= totNumBytes)
                data = uint8(fread(this.serialPort,totNumBytes)'); 
                if ~(strcmp(char(data(1:3)),'<{(')||data(4) ~= obj.md.id)||...
                       (data(end-1) ~= 13)||(data(end) ~= 10)
                    display('glo receive all failed');
                    success=0;
                    return;
                end
                len=double(obj.md.numBytes);
                starts = 8:len+9:totNumBytes;
                ends = starts+len-1;
                stuff = obj.data;
                for (i = 1:obj.md.numInstance)
                  stuff(i) = obj.parse(data(starts(i):ends(i)));
                end
                obj.data=stuff;    
                wait =0;
                success = 1;
            end
        end
    end % recieveAllObjects
    function data = attemptRecovery(this,data,obj,instance,totNumBytes)
        errorStr=sprintf('glo error: object %u, not received correctly',obj.md.id);
        n=this.serialPort.BytesAvailable;
        if n
            extradata = uint8(fread(this.serialPort,n)');
            fulldata=[data extradata];
        else
            fulldata=data;
        end
        starts = strfind(char(fulldata),'<{(');
        if isempty(starts)
            data=[];display(errorStr);
            assignin('base','gloErrorData',fulldata);return;
        end
        starts = starts(1);
        if (length(fulldata) - starts +1) < totNumBytes
            data=[];display(errorStr);
            assignin('base','gloErrorData',fulldata);return;
        end
        data = fulldata(starts:starts+totNumBytes-1);    
        if (data(4) ~= obj.md.id)||...
           (typecast(data(5:6),'uint16') ~= instance)||...
           (data(end-1) ~= 13)||(data(end) ~= 10)
              data=[];display(errorStr);
              assignin('base','gloErrorData',fulldata);return;
        end
        display('glo com message: object recovered');
    end % attemptRecovery
 end % methods
end % classdef

%snew=instrfind
%fclose(snew)