function [x,v,t] =gentrapprof(DX,Vmax,Amax,DT)
% Copyright: Dale Schinstock, 2013

%Trapezoidal-velocity motion profile generation
%
%example: [x,v,t] =trapprof(DX,Vmax,Amax,DT);
%x=postion vector, v=trapezoidal velocity vector, t=time vector
%DX=distance to move, Vmax=max' vel', Amax=max' accel', DT=time step 

%Written by DES 12/17/02

DeltaS = (Vmax^2)/Amax;  % minimum distance for a constant vel' portion

if abs(DX) < DeltaS                                %only accel and decel
   t1 = (abs(DX)/Amax)^0.5; t3 = 2*t1;                
   a1 = Amax/2; b1 = 0; c1 = 0;                       
   a3 = -Amax/2; b3 = Amax*t3; c3 = abs(DX)- 1/2*Amax*t3^2; 
   A1 = sign(DX)*[a1 b1 c1]; A3 = sign(DX)*[a3 b3 c3];
   B1 = sign(DX)*[2*a1 b1]; B3 = sign(DX)*[2*a3 b3];

   T1 = 0:DT:t1; T3 = (T1(length(T1))+DT):DT:t3;
   X1 = polyval(A1,T1); X3 = polyval(A3,T3); 
   V1 = polyval(B1,T1); V3 = polyval(B3,T3); 
   t = [T1 T3]';  x = [X1 X3]'; v = [V1 V3]';
else                                               %accel, const vel, and decel
   t1 = Vmax/Amax; t2 = abs(DX)/Vmax; t3 = t2 + t1;
   a1 = Amax/2; b1 = 0; c1 = 0;              
   a2 = 0; b2 = Amax*(2*t1*t2-t3*t1)/(t2-t1); c2 = t1^2*(a1-a2)+t1*(b1-b2)+c1;
   a3 = -Amax/2; b3 = Amax*t3; c3 = abs(DX)-1/2*Amax*t3^2; 
   A1 = sign(DX)*[a1 b1 c1]; A2 = sign(DX)*[a2 b2 c2]; A3 = sign(DX)*[a3 b3 c3];
   B1 = sign(DX)*[2*a1 b1]; B2 = sign(DX)*[2*a2 b2]; B3 = sign(DX)*[2*a3 b3];

   T1 = 0:DT:t1; T2 = (T1(length(T1))+DT):DT:t2; T3 = (T2(length(T2))+DT):DT:t3;
   X1 = polyval(A1,T1); X2 = polyval(A2,T2); X3 = polyval(A3,T3); 
   V1 = polyval(B1,T1); V2 = polyval(B2,T2); V3 = polyval(B3,T3); 
   t = [T1 T2 T3]';  x = [X1 X2 X3]'; v = [V1 V2 V3]';
end
% double the vector lengths to allow settling in simulation;
tfinal = t(length(t));
tadd= (tfinal+DT):DT:(tfinal+DT*(length(t))); 
xadd = DX*ones(size(tadd));
vadd = zeros(size(tadd));
t=[t; tadd']; x=[x; xadd']; v=[v; vadd'];
