classdef TrapProf < handle
% Copyright: Dale Schinstock, 2013

  % The only things that need to change in a glo object definition should
  % be the constant data structure below, the name of the file, the class
  % name, and the name of the constructor. The data structure should have
  % proper fields and sizes. The constructor should be the name of the class.
  
  properties (Constant)                      
      dataStruct = struct('Amax',single(1),'Vmax',single(1),'DX',single(1),...
          't1',single(1),'t2',single(1),'t3',single(1),...
          'c1',single(1:3),'c2',single(1:3),'c3',single(1:3));
  end
  properties                      
      data
  end
  properties (SetAccess=immutable)       % only changed by constructor
      md = struct('id',uint8(125),'numBytes',uint8(255),'numInstance',uint16(0));
      fields;
      types;
      parseCmdStr;
      getBytesCmdStr;
  end
  methods
      function this = TrapProf(objID,numInstances)
          dummydata = repmat(this.dataStruct,numInstances,1);
          if (objID > 254) objID = 254; end
          if (objID < 1) objID = 1; end
          objID=uint8(objID);
          if (numInstances > 65536) numInstances = 65536; end
          if (numInstances < 1) numInstances = 1; end
          numInstances=uint16(numInstances);
          
          fieldnames = fields(this.dataStruct);
          n = length(fieldnames);
          this.md.id = objID;
          this.fields = fieldnames;
          classes = cell(n,1);
          bytes = cell(n,1);
          numBytes=0;
          for(i=1:n)
             classes{i}=class(this.dataStruct.(fieldnames{i}));
             bytes{i}=length(typecast(this.dataStruct.(fieldnames{i}),'uint8'));
             numBytes = numBytes + bytes{i};
          end
          if (numBytes < 256)
              numBytes = uint8(numBytes);
          else
              error('Data structure too large');
          end
          
          this.types = cellfun(@(y)...
              arrayfun(@(x) class(x.(y)),dummydata,'UniformOutput',false)...
              ,fieldnames,'UniformOutput',false);
         
          dataPtr=1;
          ParseCmdStr = '';
          GetBytesCmdStr = 'bytes = [';
          for(i=1:n)
            ParseCmdStr = [ParseCmdStr 'dummy.' fieldnames{i}...
                '= typecast(data('...
                num2str(dataPtr) ':' num2str(dataPtr+bytes{i}-1)...
                '),''' classes{i} ''');' 10];
            GetBytesCmdStr = [GetBytesCmdStr...
                'typecast(this.data(instance).' fieldnames{i}...
                ',''uint8'') '];
            dataPtr = dataPtr+bytes{i};
          end
          GetBytesCmdStr = [GetBytesCmdStr ']' 10];
          
          this.md.numBytes = numBytes;
          this.md.numInstance = numInstances;
          this.parseCmdStr = ParseCmdStr;      
          this.getBytesCmdStr = GetBytesCmdStr;
          this.data = repmat(this.dataStruct,numInstances,1);
      end  %constructor
      
      % Note "set" will overload '=' for all members of data structure.
      % Even if you set a single member the type checking will still be
      % done here for that member, leaving all other members unchanged.
      function set.data(this,values)
          fieldnames = this.fields;
          valuestypes = cellfun(@(y)...
              arrayfun(@(x) class(x.(y)),values,'UniformOutput',false)...
              ,fieldnames,'UniformOutput',false);
          if  (~isequal(this.types,valuestypes)||~isequal(fields(values),fieldnames))
              error ('Wrong size or type for glo object')
          end
          this.data = values;
      end  % set method

      %given an array of bytes, 'data' the same length as the datastruct
      %this will return a parsed struct 'dummy'
      function dummy = parse(this,data)
        eval(this.parseCmdStr);
      end  % parse method

      %given a data struct instance 'instance', this will return
      %an array of bytes containing the data in the instance
      function bytes = getBytes(this,instance)
        eval(this.getBytesCmdStr);
      end  % parse method
  end % methods
end
