//-------------------------------------------------------------------------
//  Implementation of GloLink.
// Copyright: Dale Schinstock, 2013
//-------------------------------------------------------------------------
#include "GloLink.h"
#include "usbcdc_interface2way.h"
#include <string.h>
#include "gloObjects.h"
#include "GloOS.h"

// Constructor
GloLink::GloLink()
{
  USBCDC_Config();
}
// Destructor
GloLink::~GloLink()
{
}

// This should be called by sysTimer thread which limits one recieved object
// per sysTimer. In this case all recieved callbacks should be fast.
// Or it should be placed in main() -- i.e. the idle thread
uint8_t GloLink::checkForObjects(void)  
{                                       
  uint8_t objID=0;
  uint16_t instance=0;
  
  if (USBCDCobjPending) 
  {
    objID = USBCDCobjID;
    instance = USBCDCinstance;
    if(objID == 0)                          // an object is requested
    {
      uint8_t reqObjID = USBCDC_Data[0];
      gloOS.telemQueue.put(reqObjID,instance);
    }
    else if (objID == 255)
    {
      // TODO Ack/Nack
    }
    else   // an object has been sent
    {
      if ((objID <= NUMGLOTYPES)&&(instance <= gloObjectPtrs[objID]->md.numInstance)&&(instance>0))
      {
        uint32_t instancePtr;   
        // Do the callback if present
        if (gloObjectPtrs[objID]->md.onReceiveFuncPtr != 0)    
          gloObjectPtrs[objID]->md.onReceiveFuncPtr(instance,(void *)USBCDC_Data);
        // else copy buffer to object
        else 
        {
          instancePtr = (uint32_t)gloObjectPtrs[objID]->md.dataPtr + (uint32_t)(instance-1)*gloObjectPtrs[objID]->md.numBytes;
          __disable_irq();  // disable all interrupts
          memcpy( (void *)instancePtr, (void const *)USBCDC_Data, gloObjectPtrs[objID]->md.numBytes);
          __enable_irq(); //  enable interrupts
        }
        numReceived++;

        // NOW ACKNOWLEDGE BY SENDING THE OBJECT BACK - COSTLY, BUT SIMPLE
        gloOS.telemQueue.put(objID,instance);
      }
    }
    USBCDCobjPending=0;    // Do this last to prevent buffer from being overwritten during processing
  }
  return(objID);
}

// this should be called by a thread handling the gloOS.telemQueue
uint16_t GloLink::sendObject(uint8_t objID, uint16_t instance)
{
  uint32_t instancePtr;   
  instancePtr = (uint32_t)gloObjectPtrs[objID]->md.dataPtr +
    (uint32_t)(instance-1)*gloObjectPtrs[objID]->md.numBytes;
  
  sendBuff[0]='<'; sendBuff[1]='{'; sendBuff[2]='(';
  sendBuff[3] = objID;
  sendBuff[4] = (uint8_t)((instance<<8)>>8);
  sendBuff[5] = (uint8_t)(instance>>8);
  sendBuff[6] = gloObjectPtrs[objID]->md.numBytes;
  sendBuff[gloObjectPtrs[objID]->md.numBytes + 7] = 0x0D;  //TODO CRC if serial
  sendBuff[gloObjectPtrs[objID]->md.numBytes + 8] = 0x0A;  //CR LF
 
  __disable_irq();  // disable all interrupts
  memcpy((void *)&(sendBuff[7]), (void const *)instancePtr, gloObjectPtrs[objID]->md.numBytes);
  __enable_irq(); //  enable interrupts
  
  if (USBCDC_Send_Data(sendBuff, (uint32_t)(gloObjectPtrs[objID]->md.numBytes + 9), 1))
  {
    numSent++;
    return(1);
  }
  else
  {
    numSentFailed++;
    return(0);
  }
}