// Copyright: Dale Schinstock, 2013

#ifndef GLOOS_H
#define GLOOS_H

#include "stm32f4xx.h"
#include "gloObjects.h"
#include "Leds.h"

#define GLOOSNUMMUTEX 1
#define GLOOSNUMFLAGS 2
#define GLOOSTELEMQUEUESIZE 10

#define GLOOSDATAMUTEX 0
#define GLOOSRESETENCFLAG 0
#define GLOOSRESETINTFLAG 1

struct gloObjectQueueType{uint8_t objID; uint16_t instance;}; 

typedef enum
{
  thread0,     // systick interrupt
  thread1,     // uses EXTI_Line5 interrupt
  thread2      // uses EXTI_Line10 interrupt
} threadEnum;

typedef struct{
  __IO uint64_t ticks;
  __IO double time;
  uint16_t freq;
  float DT;
} SysTimer_t;

class GloObjectQueue
{
  public:
    explicit GloObjectQueue(uint8_t thissize);
    uint8_t put(uint8_t objID, uint16_t instance);
    uint8_t take(uint8_t *objID, uint16_t *instance);    
    uint8_t getnum(void){return num;}                //inline func
    
  private:
    gloObjectQueueType *queue;
    uint8_t num, top, bottom, size;
};

class GloOS
{
  public:
    explicit GloOS(uint16_t sysTimerFrequency,
          void (*sysTimerThread)(void),
          void (*thread1Ptr)(void),
          void (*thread2Ptr)(void));
    void delay(float seconds);                 // use to delay from anywhere except
                                               // interrupts with higher priority
                                               // than systick 
    void threadDispatcher(threadEnum thread);  // called only from stm32f4xx_it.cpp
    uint8_t mutexLock(uint8_t mutexNum);       // should not unlock unless you have locked
    void mutexUnlock(uint8_t mutexNum)
      {if(mutexNum<GLOOSNUMMUTEX) mutex[mutexNum]=0;}        
    bool flags[GLOOSNUMFLAGS];
    struct {uint32_t thread1Count; uint32_t thread2Count;} threadStats;     
    GloObjectQueue telemQueue;                 // put object on telemQueue to trigger
                                               // the telemSendThread
    SysTimer_t sysTimer;
    Leds leds;
    
  private:
    void (*threadFuncPtrs[3])(void);
    void sysTimerSetup(uint16_t SysTickFrequency);
    void threadScheduler(void);                // called by dispatcher for systick
    uint32_t thread2delay;
    uint8_t mutex[GLOOSNUMMUTEX];
};


extern GloOS gloOS;

#endif