// Copyright: Dale Schinstock, 2013

#ifndef GloObjectTemplate_H
#define GloObjectTemplate_H

#include <stdint.h>
#include "GloObjectBase.h"
#include <string.h>
#include "stm32f4xx.h"

extern GloObjectBase *gloObjectPtrs[NUMGLOTYPES+1];

template <typename objectDataType,uint16_t numInstance>
class GloObjectTemplate : public GloObjectBase
{
  public:
    //constructor
    GloObjectTemplate(uint8_t id);

    //destructor
    virtual ~GloObjectTemplate();

    //methods
    uint8_t getData(objectDataType *copy, uint16_t instance);
    uint8_t setData(objectDataType *dataStruct, uint16_t instance);
    
    //properties -- note (md) metadata is inherited from GloObjectBase
    objectDataType data[numInstance];

  private:

};

// Constructor
template <typename objectDataType,uint16_t numInstance>
GloObjectTemplate<objectDataType,numInstance>::GloObjectTemplate(uint8_t id)
{
  md.id = id;
  md.numBytes = sizeof(objectDataType);
  md.numInstance = numInstance;
  md.dataPtr = (void *)&(data);
  gloObjectPtrs[id] = this;
}

// Destructor
template <typename objectDataType,uint16_t numInstance>
GloObjectTemplate<objectDataType,numInstance>::~GloObjectTemplate()
{
}

template <typename objectDataType,uint16_t numInstance>
uint8_t GloObjectTemplate<objectDataType,numInstance>::getData(objectDataType *copy, uint16_t instance)
{
  if(instance > (md.numInstance-1)) 
     return(0);
     
  __disable_irq();  // disable all interrupts
  memcpy( (void *)copy, (void const *)&(data[instance-1]), md.numBytes);
  __enable_irq(); //  enable interrupts
  return(1);
}

template <typename objectDataType,uint16_t numInstance>
uint8_t GloObjectTemplate<objectDataType,numInstance>::setData(objectDataType *dataStruct, uint16_t instance)
{
  uint32_t instancePtr;   
  if((instance > md.numInstance)||(instance == 0))
     return(0);   
  
  instancePtr = (uint32_t)md.dataPtr + (uint32_t)(instance-1)*md.numBytes;
  
  __disable_irq();  // disable all interrupts
  memcpy( (void *)instancePtr, (void const *)dataStruct, md.numBytes);
  __enable_irq(); //  enable interrupts
  return(1);
}

// *******************************
// Do single instance specialization
// *******************************
template <typename objectDataType>
class GloObjectTemplate<objectDataType,1> : public GloObjectBase
{
  public:
    //constructor
    GloObjectTemplate(uint8_t id);

    //destructor
    virtual ~GloObjectTemplate();

    //methods
    void getData(objectDataType *copy);
    void setData(objectDataType *dataStruct);
    
    //properties -- note (md) metadata is inherited from GloObjectBase
    objectDataType data;

  private:

};

// Constructor
template <typename objectDataType>
GloObjectTemplate<objectDataType,1>::GloObjectTemplate(uint8_t id)
{
  md.id = id;
  md.numBytes = sizeof(objectDataType);
  md.numInstance = 1;
  md.dataPtr = (void *)&(data);
  gloObjectPtrs[id] = this;
}

// Destructor
template <typename objectDataType>
GloObjectTemplate<objectDataType,1>::~GloObjectTemplate()
{
}

template <typename objectDataType>
void  GloObjectTemplate<objectDataType,1>::getData(objectDataType *copy)
{
  __disable_irq();  // disable all interrupts
  memcpy( (void *)copy, (void const *)&(data), md.numBytes);
  __enable_irq(); //  enable interrupts
}

template <typename objectDataType>
void GloObjectTemplate<objectDataType,1>::setData(objectDataType *dataStruct)
{
  __disable_irq();  // disable all interrupts
  memcpy( (void *)&(data), (void const *)dataStruct, md.numBytes);
  __enable_irq(); //  enable interrupts
}

#endif  //GloObjectTemplate_H