// Copyright: Dale Schinstock, 2013

/*
glo Object format and process: 

max size of objectDataType segment is 255 bytes;
valid struct members are int or uint8,16,32, floats, bools, enums and arrays of
  those types.
if using enums or bools should make sure compiler is using 8 bits

TO DEFINE A NEW OBJECT:
  1) adjust the NUMGLOTYPES (1-254)
  2) provide the data structure below with NUM... and ID.., NUM... can be 1-65536,
     and ID is a unique for that object (up to NUMGLOTYPES)
  3) Provide appropriate definition and declaration in gloObjects.cpp and 
     gloObjects.h.
  4) write and register and appropriate onReceiveFunction, if needed.

*/

#ifndef GLOTYPES_H
#define GLOTYPES_H
#ifdef __cplusplus
//extern "C" {
#endif

#pragma pack(1)               // structures and members use and alignment of 1
  
#include <stdint.h>

#define NUMGLOTYPES 4

typedef struct{
  uint8_t id;
  uint8_t numBytes;
  uint16_t numInstance;
  void *dataPtr;
  void  (* onReceiveFuncPtr)(uint16_t instance, void *data);
} gloMetaDataType;

// Define trapProf
#define NUMTRAPPROF 1
#define TRAPPROFID 1

typedef struct{
  float Amax,Vmax,DX;
  float t1,t2,t3;
  float c1[3],c2[3],c3[3];
} trapProfDataType;

// Define logData
#define NUMLOGDATA 2048
#define LOGDATAID 2

typedef struct{
  float floatvals[9];
} logDataDataType;

// Define parameters
#define NUMPARAMETERS 1
#define PARAMETERSID 3

enum waveType_t {SINE=1, SQUARE=2, TRIANGLE=3, TRAPEZOIDAL=4};
enum controlMode_t {OPENLOOP=1, POSITION=2, SPEED=3};
enum feedbackSensor_t {MOTOR=1, LOAD=2};
enum guiCmd_t {GAINS=1,WMAG=2,WFREQ=3,WTYPE=4,CMODE=5,FSENSOR=6,AMP=7,SFREQ=8,
                STARTWAVE=9,STOPWAVE=10,STARTAUTO=11,RESETENC=12};
typedef struct{
  float Kp, Ki, Kd;
  float waveMag, waveFreq;
  waveType_t waveType;
  controlMode_t controlMode;
  feedbackSensor_t feedbackSensor;
  bool enableAmp;
  float sampleFreq;
  uint16_t sampleDelay;
  guiCmd_t guiCmd;
} parametersDataType;

// Define status
#define NUMSTATUS 1
#define STATUSID 4

enum waveState_t {STOPPED=0, CONTINUOUS=1, AUTOSAVE=2, AUTOSTART=3, AUTOPAUSE=4};
typedef struct{
  float motPos, loadPos, motSpeed, loadSpeed;
  float command, commandOffset, waveTime;
  uint16_t logDataPtr;
  waveState_t waveState;
} statusDataType;

#pragma pack()               // restore default structure alignment

#ifdef __cplusplus
//}
#endif
#endif // GLOTYPES_H


