// Copyright: Dale Schinstock, 2013

/**
  *          Main Interrupt Service Routines.
  *          This file provides template for all exceptions handler and 
  *          peripherals interrupt service routine.
  */ 

/* Includes ------------------------------------------------------------------*/
#include "stm32f4xx.h"
#include "stm32f4xx_it.h"
extern "C"{
#include "usb_core.h"
#include "usbd_core.h"
#include "usbd_cdc_core.h"
}
#include "GloOS.h"

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/

// External Variables and functions
extern USB_OTG_CORE_HANDLE           USB_OTG_dev;
extern "C" uint32_t USBD_OTG_ISR_Handler (USB_OTG_CORE_HANDLE *pdev);

/******************************************************************************/
/*            Cortex-M4 Processor Exceptions Handlers                         */
/******************************************************************************/

extern "C" void NMI_Handler(void)
{
}

extern "C" void HardFault_Handler(void)
{
  /* Go to infinite loop when Hard Fault exception occurs */
  while (1)
  {
  }
}

extern "C" void MemManage_Handler(void)
{
  /* Go to infinite loop when Memory Manage exception occurs */
  while (1)
  {
  }
}

extern "C" void BusFault_Handler(void)
{
  /* Go to infinite loop when Bus Fault exception occurs */
  while (1)
  {
  }
}

extern "C" void UsageFault_Handler(void)
{
  /* Go to infinite loop when Usage Fault exception occurs */
  while (1)
  {
  }
}

extern "C" void SVC_Handler(void)
{
}

extern "C" void DebugMon_Handler(void)
{
}

extern "C" void PendSV_Handler(void)
{
}

extern "C" void SysTick_Handler(void)
{
  gloOS.threadDispatcher(thread0);
}

extern "C" void OTG_FS_WKUP_IRQHandler(void)
{
  if(USB_OTG_dev.cfg.low_power)
  {
    /* Reset SLEEPDEEP and SLEEPONEXIT bits */
    SCB->SCR &= (uint32_t)~((uint32_t)(SCB_SCR_SLEEPDEEP_Msk | SCB_SCR_SLEEPONEXIT_Msk));
    
    /* After wake-up from sleep mode, reconfigure the system clock */
    SystemInit();
    USB_OTG_UngateClock(&USB_OTG_dev);
  }
  EXTI_ClearITPendingBit(EXTI_Line18);
}

extern "C" void OTG_FS_IRQHandler(void)
{
  USBD_OTG_ISR_Handler (&USB_OTG_dev);
}

extern "C" void EXTI9_5_IRQHandler(void)
{
  gloOS.threadDispatcher(thread1);
  
  EXTI_ClearITPendingBit(EXTI_Line5); // this clears the SW interrupt, but ...
  //EXTI->PR = EXTI_Line5;            // this results in one immediate reentry?
}

extern "C" void EXTI15_10_IRQHandler(void)
{
  gloOS.threadDispatcher(thread2);
  
  EXTI_ClearITPendingBit(EXTI_Line10); // this clears the SW interrupt, but ...
  //EXTI->PR = EXTI_Line10;            // this results in one immediate reentry?
}

extern "C" void DMA2_Stream0_IRQHandler(void)  // currently only enabled
{                                              // when testing speed of ADC
 static uint32_t count;

  if(DMA_GetITStatus(DMA2_Stream0, DMA_IT_TCIF0))
  {
    /* Clear DMA Stream Transfer Complete interrupt pending bit */
    DMA_ClearITPendingBit(DMA2_Stream0, DMA_IT_TCIF0);  
    count++;
    if (count>1200000)
      count--;
  }
}
/******************************************************************************/
/*                 STM32F4xx Peripherals Interrupt Handlers                   */
/*  Add here the Interrupt Handler for the used peripheral(s) (PPP), for the  */
/*  available peripheral interrupt handler's name please refer to the startup */
/*  file (startup_stm32f4xx.s).                                               */
/******************************************************************************/

